/*
 * fullscreen.m --
 * Implements native iOS-style fullscreen support for OS X 10.7 and later.
 * Copyright (c) 2015 Kevin Walzer/WordTech Communications LLC.
 *
 * See the file "license.terms" for information on usage and 
 redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#import <tcl.h>
#import <tk.h>
#import <tkInt.h>
#undef panic
#import <tkMacOSXInt.h>
#import <Cocoa/Cocoa.h>
#import <tkMacOSXPrivate.h>
#import <tkScrollbar.h>
#import <tkMacOSXWm.h>
#import <tkMacOSXEvent.h>
#import <tkMacOSXDebug.h>
#import <ApplicationServices/ApplicationServices.h>
#define panic Tcl_Panic

 
/*Forward declarations of variables and functions.*/
int MakeFullScreen(ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj 
		   *CONST objv[]);
int Fullscreen_Init (Tcl_Interp *interp);
int Fullscreen_SafeInit(Tcl_Interp *interp);
TkWindow* TkMacOSXGetTkWindow(NSWindow *w);

/*From Tk source code: get a Tk window from an NSWindow instance.*/
TkWindow* TkMacOSXGetTkWindow(NSWindow *w)  {
  Window window = TkMacOSXGetXWindow(w);
  TkDisplay *dispPtr = TkGetDisplayList();

  return (window != None ? (TkWindow *)Tk_IdToWindow(dispPtr->display, 
						     window) :
	  NULL);
};


/*Category on NSWindow to intercept toggleFullScreen call, generate virtual event for processing at script level.*/
@interface NSWindow(TkFullScreen)
-(void) toggleFullScreen:(id) sender;
@end

@implementation NSWindow(TkFullScreen)

- (void)toggleFullScreen:(id)sender
{

  XVirtualEvent event;
  int x, y;
  TkWindow *winPtr = TkMacOSXGetTkWindow(self);
  Tk_Window tkwin = (Tk_Window) winPtr;

  bzero(&event, sizeof(XVirtualEvent));
  event.type = VirtualEvent;
  event.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
  event.send_event = false;
  event.display = Tk_Display(tkwin);
  event.event = Tk_WindowId(tkwin);
  event.root = XRootWindow(Tk_Display(tkwin), 0);
  event.subwindow = None;
  event.time = TkpGetMS();
  XQueryPointer(NULL, winPtr->window, NULL, NULL,
		&event.x_root, &event.y_root, &x, &y, &event.state);
  Tk_TopCoordsToWindow(tkwin, x, y, &event.x, &event.y);
  event.same_screen = true;
  event.name = Tk_GetUid("ToggleFullScreen");
  Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
}

@end


/*Tcl function to add fullscreen button.*/
int MakeFullScreen(ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj 
		   *CONST objv[]) {

  if(objc != 2) {
    Tcl_WrongNumArgs(ip, 1, objv, "path?");
    return TCL_ERROR;
  }

  /* Get the object that holds this Tk Window.*/

  Tk_Window path;
  path = Tk_NameToWindow(ip, Tcl_GetString(objv[1]), Tk_MainWindow(ip));
  if (path == NULL) {
    return TCL_ERROR;
  }

  Tk_MakeWindowExist(path);
  Tk_MapWindow(path);
  Drawable d = Tk_WindowId(path);

  /*Get NSView from Tk window.*/
  NSView *view = TkMacOSXGetRootControl(d);

  NSWindow *window = [view window];	
  [window setCollectionBehavior:NSWindowCollectionBehaviorFullScreenPrimary];
  return TCL_OK;
}


/*Initalize the package in the tcl interpreter, create Tcl commands. */
int Fullscreen_Init (Tcl_Interp *interp) {


  if (Tcl_InitStubs(interp, "8.5", 0) == NULL) {
    return TCL_ERROR;
  }
  if (Tk_InitStubs(interp, "8.5", 0) == NULL) {
    return TCL_ERROR;
  }


  Tcl_CreateObjCommand(interp, "::fullscreen::addfullscreenbutton", 
		       MakeFullScreen,(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  if (Tcl_PkgProvide(interp, "fullscreen", "1.2") != TCL_OK) {
    return TCL_ERROR;
  }

  return TCL_OK;
	
}

int Fullscreen_SafeInit(Tcl_Interp *interp) {
  return Fullscreen_Init(interp);
}




